@@ -63,21 +63,35 @@ def ADD_LD_LIBRARY_PATH(path):
         os.environ['LD_LIBRARY_PATH'] = ':'.join(newpath)
 
 
-def install_rpath(bld):
+def needs_private_lib(bld, target):
+    '''return True if a target links to a private library'''
+    for lib in getattr(target, "uselib_local", []):
+        t = bld.name_to_obj(lib, bld.env)
+        if t and getattr(t, 'private_library', False):
+            return True
+    return False
+
+
+def install_rpath(target):
     '''the rpath value for installation'''
+    bld = target.bld
     bld.env['RPATH'] = []
+    ret = set()
     if bld.env.RPATH_ON_INSTALL:
-        return ['%s/lib' % bld.env.PREFIX]
-    return []
+        ret.add(bld.EXPAND_VARIABLES(bld.env.LIBDIR))
+    if bld.env.RPATH_ON_INSTALL_PRIVATE and needs_private_lib(bld, target):
+        ret.add(bld.EXPAND_VARIABLES(bld.env.PRIVATELIBDIR))
+    return list(ret)
 
 
 def build_rpath(bld):
     '''the rpath value for build'''
-    rpath = os.path.normpath('%s/%s' % (bld.env.BUILD_DIRECTORY, LIB_PATH))
+    rpaths = [os.path.normpath('%s/%s' % (bld.env.BUILD_DIRECTORY, d)) for d in ("shared", "shared/private")]
     bld.env['RPATH'] = []
     if bld.env.RPATH_ON_BUILD:
-        return [rpath]
-    ADD_LD_LIBRARY_PATH(rpath)
+        return rpaths
+    for rpath in rpaths:
+        ADD_LD_LIBRARY_PATH(rpath)
     return []
 
 
@@ -102,8 +116,7 @@ def LOCAL_CACHE_SET(ctx, cachename, key, value):
 def ASSERT(ctx, expression, msg):
     '''a build assert call'''
     if not expression:
-        Logs.error("ERROR: %s\n" % msg)
-        raise AssertionError
+        raise Utils.WafError("ERROR: %s\n" % msg)
 Build.BuildContext.ASSERT = ASSERT
 
 
@@ -295,7 +308,7 @@ def recursive_dirlist(dir, relbase, pattern=None):
 
 def mkdir_p(dir):
     '''like mkdir -p'''
-    if os.path.isdir(dir):
+    if not dir or os.path.isdir(dir):
         return
     mkdir_p(os.path.dirname(dir))
     os.mkdir(dir)
@@ -326,6 +339,9 @@ def EXPAND_VARIABLES(ctx, varstr, vars=None):
             ret.append(EXPAND_VARIABLES(ctx, s, vars=vars))
         return ret
 
+    if not isinstance(varstr, str):
+        return varstr
+
     import Environment
     env = Environment.Environment()
     ret = varstr
@@ -527,3 +543,64 @@ def reconfigure(ctx):
     bld = samba_wildcard.fake_build_environment()
     Configure.autoconfig = True
     Scripting.check_configured(bld)
+
+
+def map_shlib_extension(ctx, name, python=False):
+    '''map a filename with a shared library extension of .so to the real shlib name'''
+    if name is None:
+        return None
+    if name[-1:].isdigit():
+        # some libraries have specified versions in the wscript rule
+        return name
+    (root1, ext1) = os.path.splitext(name)
+    if python:
+        (root2, ext2) = os.path.splitext(ctx.env.pyext_PATTERN)
+    else:
+        (root2, ext2) = os.path.splitext(ctx.env.shlib_PATTERN)
+    return root1+ext2
+Build.BuildContext.map_shlib_extension = map_shlib_extension
+
+
+def make_libname(ctx, name, nolibprefix=False, version=None, python=False):
+    """make a library filename
+         Options:
+              nolibprefix: don't include the lib prefix
+              version    : add a version number
+              python     : if we should use python module name conventions"""
+
+    if python:
+        libname = ctx.env.pyext_PATTERN % name
+    else:
+        libname = ctx.env.shlib_PATTERN % name
+    if nolibprefix and libname[0:3] == 'lib':
+        libname = libname[3:]
+    if version:
+        if version[0] == '.':
+            version = version[1:]
+        (root, ext) = os.path.splitext(libname)
+        if ext == ".dylib":
+            # special case - version goes before the prefix
+            libname = "%s.%s%s" % (root, version, ext)
+        else:
+            libname = "%s%s.%s" % (root, ext, version)
+    return libname
+Build.BuildContext.make_libname = make_libname
+
+
+def get_tgt_list(bld):
+    '''return a list of build objects for samba'''
+
+    targets = LOCAL_CACHE(bld, 'TARGET_TYPE')
+
+    # build a list of task generators we are interested in
+    tgt_list = []
+    for tgt in targets:
+        type = targets[tgt]
+        if not type in ['SUBSYSTEM', 'MODULE', 'BINARY', 'LIBRARY', 'ASN1', 'PYTHON']:
+            continue
+        t = bld.name_to_obj(tgt, bld.env)
+        if t is None:
+            Logs.error("Target %s of type %s has no task generator" % (tgt, type))
+            sys.exit(1)
+        tgt_list.append(t)
+    return tgt_list
