@@ -1,7 +1,7 @@
 # a waf tool to add autoconf-like macros to the configure section
 # and for SAMBA_ macros for building libraries, binaries etc
 
-import Build, os, Options, Task, Utils, cc, TaskGen, fnmatch, re, shutil, Logs, Constants
+import Build, os, sys, Options, Task, Utils, cc, TaskGen, fnmatch, re, shutil, Logs, Constants
 from Configure import conf
 from Logs import debug
 from samba_utils import SUBST_VARS_RECURSIVE
@@ -23,10 +23,13 @@ import samba_conftests
 import samba_abi
 import tru64cc
 import irixcc
+import hpuxcc
 import generic_cc
 import samba_dist
 import samba_wildcard
 import stale_files
+import symbols
+import pkgconfig
 
 # some systems have broken threading in python
 if os.environ.get('WAF_NOTHREADS') == '1':
@@ -54,6 +57,7 @@ def SAMBA_BUILD_ENV(conf):
     '''create the samba build environment'''
     conf.env.BUILD_DIRECTORY = conf.blddir
     mkdir_p(os.path.join(conf.blddir, LIB_PATH))
+    mkdir_p(os.path.join(conf.blddir, LIB_PATH, "private"))
     mkdir_p(os.path.join(conf.blddir, "modules"))
     mkdir_p(os.path.join(conf.blddir, 'python/samba/dcerpc'))
     # this allows all of the bin/shared and bin/python targets
@@ -97,25 +101,29 @@ def SAMBA_LIBRARY(bld, libname, source,
                   header_path=None,
                   pc_files=None,
                   vnum=None,
+                  soname=None,
                   cflags='',
+                  ldflags='',
                   external_library=False,
                   realname=None,
                   autoproto=None,
-                  group='main',
+                  group='libraries',
                   depends_on='',
                   local_include=True,
                   vars=None,
                   install_path=None,
                   install=True,
                   pyembed=False,
+                  pyext=False,
                   target_type='LIBRARY',
                   bundled_extension=True,
                   link_name=None,
-                  abi_file=None,
+                  abi_directory=None,
                   abi_match=None,
                   hide_symbols=False,
-                  is_bundled=False,
                   manpages=None,
+                  private_library=False,
+                  grouping_library=False,
                   enabled=True):
     '''define a Samba library'''
 
@@ -135,6 +143,11 @@ def SAMBA_LIBRARY(bld, libname, source,
     else:
         obj_target = libname + '.objlist'
 
+    if group == 'libraries':
+        subsystem_group = 'main'
+    else:
+        subsystem_group = group
+
     # first create a target for building the object files for this library
     # by separating in this way, we avoid recompiling the C files
     # separately for the install library and the build library
@@ -146,11 +159,11 @@ def SAMBA_LIBRARY(bld, libname, source,
                         public_headers = public_headers,
                         header_path    = header_path,
                         cflags         = cflags,
-                        group          = group,
+                        group          = subsystem_group,
                         autoproto      = autoproto,
                         depends_on     = depends_on,
                         hide_symbols   = hide_symbols,
-                        pyext          = (target_type == "PYTHON"),
+                        pyext          = pyext or (target_type == "PYTHON"),
                         local_include  = local_include)
 
     if BUILTIN_LIBRARY(bld, libname):
@@ -164,44 +177,73 @@ def SAMBA_LIBRARY(bld, libname, source,
     deps = TO_LIST(deps)
     deps.append(obj_target)
 
-    if target_type == 'PYTHON' or realname or not is_bundled:
-        # Sanitize the library name
-        bundled_name = libname.lower().replace('_', '-')
-        while bundled_name.startswith("lib"):
-            bundled_name = bundled_name[3:]
+    realname = bld.map_shlib_extension(realname, python=(target_type=='PYTHON'))
+    link_name = bld.map_shlib_extension(link_name, python=(target_type=='PYTHON'))
+
+    # we don't want any public libraries without version numbers
+    if not private_library and vnum is None and soname is None and target_type != 'PYTHON' and not realname:
+        raise Utils.WafError("public library '%s' must have a vnum" % libname)
+
+    if target_type == 'PYTHON' or realname or not private_library:
+        bundled_name = libname.replace('_', '-')
     else:
-        bundled_name = BUNDLED_NAME(bld, libname, bundled_extension)
+        bundled_name = PRIVATE_NAME(bld, libname, bundled_extension, private_library)
+
+    ldflags = TO_LIST(ldflags)
 
     features = 'cc cshlib symlink_lib install_lib'
     if target_type == 'PYTHON':
         features += ' pyext'
-    if pyembed:
+    if pyext or pyembed:
+        # this is quite strange. we should add pyext feature for pyext
+        # but that breaks the build. This may be a bug in the waf python tool
         features += ' pyembed'
-    if abi_file:
+
+    if abi_directory:
         features += ' abi_check'
 
-    if abi_file:
-        abi_file = os.path.join(bld.curdir, abi_file)
+    vscript = None
+    if bld.env.HAVE_LD_VERSION_SCRIPT:
+        if private_library:
+            version = "%s_%s" % (Utils.g_module.APPNAME, Utils.g_module.VERSION)
+        elif vnum:
+            version = "%s_%s" % (libname, vnum)
+        else:
+            version = None
+        if version:
+            vscript = "%s.vscript" % libname
+            bld.ABI_VSCRIPT(libname, abi_directory, version, vscript,
+                            abi_match)
+            fullname = bld.env.shlib_PATTERN % bundled_name
+            bld.add_manual_dependency(bld.path.find_or_declare(fullname), bld.path.find_or_declare(vscript))
+            if Options.is_install:
+                # also make the .inst file depend on the vscript
+                instname = bld.env.shlib_PATTERN % (bundled_name + '.inst')
+                bld.add_manual_dependency(bld.path.find_or_declare(instname), bld.path.find_or_declare(vscript))
+            vscript = os.path.join(bld.path.abspath(bld.env), vscript)
 
     bld.SET_BUILD_GROUP(group)
     t = bld(
         features        = features,
         source          = [],
         target          = bundled_name,
-        samba_cflags    = CURRENT_CFLAGS(bld, libname, cflags),
         depends_on      = depends_on,
+        samba_ldflags   = ldflags,
         samba_deps      = deps,
         samba_includes  = includes,
+        version_script  = vscript,
         local_include   = local_include,
         vnum            = vnum,
+        soname          = soname,
         install_path    = None,
         samba_inst_path = install_path,
         name            = libname,
         samba_realname  = realname,
         samba_install   = install,
-        abi_file        = abi_file,
+        abi_directory   = "%s/%s" % (bld.path.abspath(), abi_directory),
         abi_match       = abi_match,
-        is_bundled      = is_bundled
+        private_library = private_library,
+        grouping_library=grouping_library
         )
 
     if realname and not link_name:
@@ -260,6 +302,11 @@ def SAMBA_BINARY(bld, binname, source,
     source = bld.EXPAND_VARIABLES(source, vars=vars)
     source = unique_list(TO_LIST(source))
 
+    if group == 'binaries':
+        subsystem_group = 'main'
+    else:
+        subsystem_group = group
+
     # first create a target for building the object files for this binary
     # by separating in this way, we avoid recompiling the C files
     # separately for the install binary and the build binary
@@ -268,7 +315,7 @@ def SAMBA_BINARY(bld, binname, source,
                         deps           = deps,
                         includes       = includes,
                         cflags         = cflags,
-                        group          = group,
+                        group          = subsystem_group,
                         autoproto      = autoproto,
                         subsystem_name = subsystem_name,
                         local_include  = local_include,
@@ -286,7 +333,6 @@ def SAMBA_BINARY(bld, binname, source,
         features       = features,
         source         = [],
         target         = binname,
-        samba_cflags   = CURRENT_CFLAGS(bld, binname, cflags),
         samba_deps     = deps,
         samba_includes = includes,
         local_include  = local_include,
@@ -295,15 +341,10 @@ def SAMBA_BINARY(bld, binname, source,
         samba_subsystem= subsystem_name,
         install_path   = None,
         samba_inst_path= install_path,
-        samba_install  = install
+        samba_install  = install,
+        samba_ldflags  = TO_LIST(ldflags)
         )
 
-    # setup the subsystem_name as an alias for the real
-    # binary name, so it can be found when expanding
-    # subsystem dependencies
-    if subsystem_name is not None:
-        bld.TARGET_ALIAS(subsystem_name, binname)
-
     if manpages is not None and 'XSLTPROC_MANPAGES' in bld.env and bld.env['XSLTPROC_MANPAGES']:
         bld.MANPAGES(manpages)
 
@@ -316,25 +357,21 @@ def SAMBA_MODULE(bld, modname, source,
                  includes='',
                  subsystem=None,
                  init_function=None,
+                 module_init_name='samba_init_module',
                  autoproto=None,
                  autoproto_extra_source='',
-                 aliases=None,
                  cflags='',
                  internal_module=True,
                  local_include=True,
                  vars=None,
                  enabled=True,
-                 pyembed=True,
+                 pyembed=False,
                  ):
     '''define a Samba module.'''
 
     source = bld.EXPAND_VARIABLES(source, vars=vars)
 
     if internal_module or BUILTIN_LIBRARY(bld, modname):
-        # treat internal modules as subsystems for now
-        if subsystem is not None:
-            deps += ' ' + subsystem
-
         bld.SAMBA_SUBSYSTEM(modname, source,
                     deps=deps,
                     includes=includes,
@@ -351,39 +388,38 @@ def SAMBA_MODULE(bld, modname, source,
         SET_TARGET_TYPE(bld, modname, 'DISABLED')
         return
 
-    modnames = [modname] + TO_LIST(aliases)
-    for modname in modnames:
-        obj_target = modname + '.objlist'
+    obj_target = modname + '.objlist'
 
-        realname = modname
-        if subsystem is not None:
-            deps += ' ' + subsystem
-            while realname.startswith("lib"+subsystem+"_"):
-                realname = realname[len("lib"+subsystem+"_"):]
-            while realname.startswith(subsystem+"_"):
-                realname = realname[len(subsystem+"_"):]
+    realname = modname
+    if subsystem is not None:
+        deps += ' ' + subsystem
+        while realname.startswith("lib"+subsystem+"_"):
+            realname = realname[len("lib"+subsystem+"_"):]
+        while realname.startswith(subsystem+"_"):
+            realname = realname[len(subsystem+"_"):]
 
-        realname = bld.env.shlib_PATTERN % realname
-        while realname.startswith("lib"):
-            realname = realname[len("lib"):]
+    realname = bld.make_libname(realname)
+    while realname.startswith("lib"):
+        realname = realname[len("lib"):]
 
-        build_link_name = "modules/%s/%s" % (subsystem, realname)
+    build_link_name = "modules/%s/%s" % (subsystem, realname)
 
-        if init_function:
-            cflags += " -D%s=samba_init_module" % init_function
+    if init_function:
+        cflags += " -D%s=%s" % (init_function, module_init_name)
+
+    bld.SAMBA_LIBRARY(modname,
+                      source,
+                      deps=deps,
+                      cflags=cflags,
+                      realname = realname,
+                      autoproto = autoproto,
+                      local_include=local_include,
+                      vars=vars,
+                      link_name=build_link_name,
+                      install_path="${MODULESDIR}/%s" % subsystem,
+                      pyembed=pyembed,
+                      )
 
-        bld.SAMBA_LIBRARY(modname,
-                          source,
-                          deps=deps,
-                          cflags=cflags,
-                          realname = realname,
-                          autoproto = autoproto,
-                          local_include=local_include,
-                          vars=vars,
-                          link_name=build_link_name,
-                          install_path="${MODULESDIR}/%s" % subsystem,
-                          pyembed=pyembed,
-                          )
 
 Build.BuildContext.SAMBA_MODULE = SAMBA_MODULE
 
@@ -487,6 +523,8 @@ def SAMBA_GENERATOR(bld, name, rule, source='', target='',
         on_results=True,
         before='cc',
         ext_out='.c',
+        samba_type='GENERATOR',
+        vars = [rule],
         name=name)
 
     if always:
@@ -507,6 +545,7 @@ def SETUP_BUILD_GROUPS(bld):
     bld.env['USING_BUILD_GROUPS'] = True
     bld.add_group('setup')
     bld.add_group('build_compiler_source')
+    bld.add_group('vscripts')
     bld.add_group('base_libraries')
     bld.add_group('generators')
     bld.add_group('compiler_prototypes')
@@ -515,7 +554,10 @@ def SETUP_BUILD_GROUPS(bld):
     bld.add_group('build_source')
     bld.add_group('prototypes')
     bld.add_group('main')
+    bld.add_group('symbolcheck')
+    bld.add_group('libraries')
     bld.add_group('binaries')
+    bld.add_group('syslibcheck')
     bld.add_group('final')
 Build.BuildContext.SETUP_BUILD_GROUPS = SETUP_BUILD_GROUPS
 
@@ -546,7 +588,7 @@ def ENABLE_TIMESTAMP_DEPENDENCIES(conf):
 
 
 
-t = Task.simple_task_type('copy_script', 'rm -f ${LINK_TARGET} && ln -s ${SRC[0].abspath(env)} ${LINK_TARGET}',
+t = Task.simple_task_type('copy_script', 'rm -f "${LINK_TARGET}" && ln -s "${SRC[0].abspath(env)}" ${LINK_TARGET}',
                           shell=True, color='PINK', ext_in='.bin')
 t.quiet = True
 
@@ -801,63 +843,6 @@ def PUBLIC_HEADERS(bld, public_headers, header_path=None):
 Build.BuildContext.PUBLIC_HEADERS = PUBLIC_HEADERS
 
 
-def subst_at_vars(task):
-    '''substiture @VAR@ style variables in a file'''
-    src = task.inputs[0].srcpath(task.env)
-    tgt = task.outputs[0].bldpath(task.env)
-
-    f = open(src, 'r')
-    s = f.read()
-    f.close()
-    # split on the vars
-    a = re.split('(@\w+@)', s)
-    out = []
-    done_var = {}
-    back_sub = [ ('PREFIX', '${prefix}'), ('EXEC_PREFIX', '${exec_prefix}')]
-    for v in a:
-        if re.match('@\w+@', v):
-            vname = v[1:-1]
-            if not vname in task.env and vname.upper() in task.env:
-                vname = vname.upper()
-            if not vname in task.env:
-                Logs.error("Unknown substitution %s in %s" % (v, task.name))
-                sys.exit(1)
-            v = SUBST_VARS_RECURSIVE(task.env[vname], task.env)
-            # now we back substitute the allowed pc vars
-            for (b, m) in back_sub:
-                s = task.env[b]
-                if s == v[0:len(s)]:
-                    if not b in done_var:
-                        # we don't want to substitute the first usage
-                        done_var[b] = True
-                    else:
-                        v = m + v[len(s):]
-                    break
-        out.append(v)
-    contents = ''.join(out)
-    f = open(tgt, 'w')
-    s = f.write(contents)
-    f.close()
-    return 0
-
-
-
-def PKG_CONFIG_FILES(bld, pc_files, vnum=None):
-    '''install some pkg_config pc files'''
-    dest = '${PKGCONFIGDIR}'
-    dest = bld.EXPAND_VARIABLES(dest)
-    for f in TO_LIST(pc_files):
-        base=os.path.basename(f)
-        t = bld.SAMBA_GENERATOR('PKGCONFIG_%s' % base,
-                                rule=subst_at_vars,
-                                source=f+'.in',
-                                target=f)
-        if vnum:
-            t.env.PACKAGE_VERSION = vnum
-        INSTALL_FILES(bld, dest, f, flat=True, destname=base)
-Build.BuildContext.PKG_CONFIG_FILES = PKG_CONFIG_FILES
-
-
 def MANPAGES(bld, manpages):
     '''build and install manual pages'''
     bld.env.MAN_XSL = 'http://docbook.sourceforge.net/release/xsl/current/manpages/docbook.xsl'
@@ -903,6 +888,9 @@ def samba_display(self):
         if target_type in type_map:
             return progress_display(self, type_map[target_type], self.name)
 
+    if len(self.inputs) == 0:
+        return Task.Task.old_display(self)
+
     fname = self.inputs[0].bldpath(self.env)
     if fname[0:3] == '../':
         fname = fname[3:]
@@ -921,3 +909,15 @@ def samba_display(self):
 
 Task.TaskBase.classes['Task'].old_display = Task.TaskBase.classes['Task'].display
 Task.TaskBase.classes['Task'].display = samba_display
+
+
+@after('apply_link')
+@feature('cshlib')
+def apply_bundle_remove_dynamiclib_patch(self):
+    if self.env['MACBUNDLE'] or getattr(self,'mac_bundle',False):
+        if not getattr(self,'vnum',None):
+            try:
+                self.env['LINKFLAGS'].remove('-dynamiclib')
+                self.env['LINKFLAGS'].remove('-single_module')
+            except ValueError:
+                pass
